function S = SparseCodes(data, W, Wg, lambda, nBasis, nIters)
% This function calculates the final sparse representation of data by graph
% based sparse coding and max pooling.
%
% input:
% --data:      size: m x n x c. m - samples, n - normalized frames, c - feature.
% --W:         graph weight matrix.
% --Wg:        parameter for graph regularization, default:1 
% --lambda:    parameter for sparsity, default:0.1
% --nBasis:    size of dictionary, default:128
% --nIters:    number of iteration, default:5
%
% output:
% --S:         sparse codes of data. size: m x nBasis.       

    tic;

    if (~exist('W','var'))
       W = buildGraph(data, 'L1Graph',5);
    end

    if (~exist('Wg','var'))
       Wg = 1;
    end

    if (~exist('lambda','var'))
       lambda = 0.1;
    end

    if (~exist('nBasis','var'))
       nBasis = 128;
    end

    if (~exist('nIters','var'))
       nBasis = 5;
    end

    % data
    [m,n,c] = size(data);

    % reshape data for training
    fea = zeros(m*n,c);
    for i = 1:m
        for j = 1:n
            fea((i-1)*n+j,1:c) = data(i,j,1:c);
        end
    end

    % training
    warning('off', 'all');
    disp(['Sparse codes training begins!']);
    rng('default');
    [B, nS, stat] = GraphSC(fea', W, nBasis, Wg, lambda, nIters); 
    disp(['Sparse codes training ends!']);

    % max pooling
    S = zeros(m,nBasis);
    for i = 1:m
       for j = 1:nBasis
           S(i,j) = max(nS(j,(i-1)*n+1:i*n));
       end
    end

    toc;
end