function Rate = Train_Recog(database, mode)
% This is the main function of the proposed algorithm.
% 
% input:
% --database:  database used for training and recognization. choices are:
%              - 'CASME' (default)
%              - 'CASMEII'
%              - 'SMIC'
%              
% --mode:      training model used for micro-expression. choices are:
%              - 'BasicSC-MDMO' 
%              - 'GraphSC-MDMO'
%              - 'Sparse MDMO' (default)
% 
% output:
% --Rate:      Rate of micro-expression recognition.

    if (~exist('database','var'))
       database = 'CASME';
    end
    
    if (~exist('mode','var'))
       mode = 'Sparse MDMO';
    end
    
    %load database information and parameter
    dbid = 1;
    mdid = 1;
    
    if strcmp(database, 'CASME') == 1
        load('casme.mat');
        dbid = 1;
    elseif strcmp(database, 'CASMEII') == 1
        load('casme2.mat');
        dbid = 2;
    else
        load('smic.mat');
        dbid = 3;
    end
    
    load('data_para.mat');
    if strcmp(mode, 'BasicSC-MDMO') == 1
        mdid = 1;
        para = data_para((dbid-1)*3+1);
    elseif strcmp(mode, 'GraphSC-MDMO') == 1
        mdid = 2;
        para = data_para((dbid-1)*3+2);
    else
        mdid = 3;
        para = data_para((dbid-1)*3+3);
    end
        
    nSamples = size(data_stru,2);
    MDMOfea = zeros(nSamples,63,72);
    subject = zeros(nSamples,1);
    labels = zeros(nSamples, 1);
    for i=1:nSamples
        if ischar(data_stru(i).subject)
            subject(i,1) = str2double(data_stru(i).subject);
        else
            subject(i,1) = data_stru(i).subject;
        end
        labels(i,1)  = data_stru(i).label;
        MDMOfea(i,:,:) = data_stru(i).MDMOfea;
    end
        
    % balance magnitude and orientation using a weight of 0.9
    [m,n,c] = size(MDMOfea);
    for i = 1:m
        for j = 1:n
            for k = 1:c/2
                MDMOfea(i,j,k) = MDMOfea(i,j,k)*0.9;
                MDMOfea(i,j,k+36) = MDMOfea(i,j,k+36)*0.1;
            end
        end
    end
    
    % buildGraph
    if mdid == 2
        W = buildGraph(MDMOfea, 'Euclidean', 5);
    else
        % for GraphSC-MDMO the following Wg is set to 0
        W = buildGraph(MDMOfea, 'L1Graph', 5);
    end

    % training
    S = SparseCodes(MDMOfea,W,para(1).Wg,para(1).lambda,para(1).dictionary_size,para(1).iters);
    
    % recognization
    disp(['Recognizing micro-expression...']);
    Rate = LOSO(S,labels,subject,para(1).degree,para(1).roef,para(1).gamma);
    
    disp(['Micro-expreesion recognition rate of ' mode ' in ' database ' is: ' num2str(Rate,'%.2f') '%.']);
end